#include <CONIO.H>
#include <DOS.H>
#include <STRING.H>
#include <STDLIB.H>




unsigned long       BaudRate    = 57600;
unsigned char       ComPort     = 1;
unsigned char       COM_Error   = 0;
unsigned char       Success     = 0;
unsigned char       StationNum  = 0;
unsigned char       StationList[32] = {0, 0, 0, 0, 0, 0, 0, 0,
                                       0, 0, 0, 0, 0, 0, 0, 0,
                                       0, 0, 0, 0, 0, 0, 0, 0,
                                       0, 0, 0, 0, 0, 0, 0, 0 };



const unsigned char        ERR_NONE       = 0x00;

const unsigned char        ERR_INITCOM    = 0x01;
const unsigned char        ERR_TIMEOUT    = 0x02;
const unsigned char        ERR_TIMEOUTTX  = 0x03;
const unsigned char        ERR_TIMEOUTRX  = 0x04;
const unsigned char        ERR_RXSUM      = 0x05;
const unsigned char        ERR_DATASIZE   = 0x06;
const unsigned char        ERR_ADDRESS    = 0x07;
const unsigned char        ERR_DEBUGFLAG  = 0x08;
const unsigned char        ERR_MOREBAUD   = 0x09;
const unsigned char        ERR_NETEMPTY   = 0x0A;

const unsigned char        ERR_BADERROR   = 0x80;
const unsigned char        ERR_BADCMD     = 0x81;
const unsigned char        ERR_TARGETSUM  = 0x82;
const unsigned char        ERR_BADFRAME   = 0x83;
const unsigned char        ERR_SELFRX     = 0x84;
const unsigned char        ERR_IOFULL     = 0x85;
const unsigned char        ERR_NODATA     = 0x86;



#define MAX_INTERNERR   10
#define MAX_TARGETERR   6

  const char            *ErrText_IPrefix = "INTERNAL ERROR: ";
  const char            *ErrText_TPrefix = "  TARGET ERROR: ";

  const char            *ErrText_Internal[] = {
    "UNSUPPORTED ERROR CODE",
    "COM-PORT INIT ERROR",
    "GENERAL TIME-OUT ON COM",
    "TX TIME-OUT ON COM OVERFLOW",
    "RX TIME-OUT ON COM OVERFLOW",
    "CHECKSUM ERROR ON RX DATA",
    "RX BLOCK LENGTH NOT SPECIFIED",
    "STATION ADDRESS OUT OF RANGE",
    "DEBUG - MARKING FLAG",
    "MULTIPLE BAUD RATES ON NETWORK",
    "NO STATIONS ON NETWORK"
  };


  const char            *ErrText_Target[] = {
    "UNSUPPORTED ERROR CODE",
    "UNKNOWN COMMAND IN CMD-FRAME",
    "CHECKSUM ERROR IN LAST FRAME",
    "UNKNOWN TYPE OF FRAME",
    "TRANSMITT SELF-CHECK ERROR",
    "INPUT BUFFER FULL - FRAME LOST",
    "IO BUFFER EMPTY !"
  };


#define STD_TIMEOUT     2

#define LINE_RXTX       0x3F
#define LINE_TXHEAD     0x37
#define LINE_DLAB       0xDD
#define LINE_BREAK      0x7D

#define DOUBLESTOP      0x04
#define PAR_ENABLE      0x08
#define PAR_EVEN        0x10
#define PAR_NULL        0x20
#define BREAK           0x40
#define DLAB            0x80


#define DEFAULT_MCTRL   0x01

#define DTR             0x01
#define RTS             0x02
#define IRQ_ENABLE      0x08


#define STAT_RXREADY    0x01
#define STAT_OVERRUN    0x02
#define STAT_ERRPARITY  0x04
#define STAT_ERRFRAMING 0x08
#define STAT_TXREADY    0x20
#define STAT_TXEMPTY    0x40

#define HD_CMDRQ        0xA0
#define HD_CMDF         0x80
#define HD_TOKEN        0xC0
#define HD_REQUEST      0xE0


  unsigned long         *p_BiosTime = (unsigned long *)(0, 0x046C);
  unsigned long         WDog;


  unsigned char         ModemCtrl   = DEFAULT_MCTRL;

  unsigned char         COM_Lock    = 1;
  unsigned char         COM_Disable = 1;

  unsigned char         LineStatus;
  unsigned char         LastLineStatus;

  unsigned int          UART_DATA;
  unsigned int          UART_LDIV;
  unsigned int          UART_IE;
  unsigned int          UART_HDIV;
  unsigned int          UART_ID;
  unsigned int          UART_LCTRL;
  unsigned int          UART_MCTRL;
  unsigned int          UART_LSTAT;
  unsigned int          UART_MSTAT;

  unsigned char         AddrTarget  = 0;

  unsigned char         IOData[21];
  unsigned char         IndirectLen = 0;


typedef struct {

  unsigned char OK;
  unsigned int  FWVersion;
  unsigned int  StackAddr;
  unsigned int  MemTop;

} InfoType;

  InfoType              StationInfo[32];


const char              BdNum = 4;
const unsigned long     BdList[4] = {2400, 9600, 19200, 57600};


#define NUM_REPEATS     3
#define STPBUFFER_SIZE  5120
#define STUPID_BEGIN    0x0400
#define ACK_KEYWORD     0x5A




typedef struct {

  unsigned char Cm;
  unsigned char Tx;
  unsigned char Rx;

} CmType;





// DIRECT COMMANDS
//-----------------
  const CmType          Cm_GetACK      = {0x01, 0, 1};

  const CmType          Cm_GetStatus   = { 0x02,  0, 1};
  const CmType          Cm_GetNETVer   = { 0x03,  0, 6};
  const CmType          Cm_GetSTPVer   = { 0x04,  0, 6};

  const CmType          Cm_WriteXByte  = { 0x05,  3, 0};
  const CmType          Cm_ReadXByte   = { 0x06,  2, 1};

  const CmType          Cm_Abort       = { 0x07,  0, 0};

  const CmType          Cm_SetXPtr     = { 0x08,  2, 0};

  const CmType          Cm_WriteXPB    = { 0x09,  1, 0};
  const CmType          Cm_WriteXPW    = { 0x0A,  2, 0};
  const CmType          Cm_WriteXPDW   = { 0x0B,  4, 0};

  const CmType          Cm_ReadXPB     = { 0x0C,  0, 1};
  const CmType          Cm_ReadXPW     = { 0x0D,  0, 2};
  const CmType          Cm_ReadXPDW    = { 0x0E,  0, 4};

  const CmType          Cm_WriteSWord  = { 0x0F,  3, 0};
  const CmType          Cm_ReadSWord   = { 0x10,  1, 2};

  const CmType          Cm_SetSBit     = { 0x11,  2, 0};
  const CmType          Cm_ClrSBit     = { 0x12,  2, 0};
  const CmType          Cm_ReadSBit    = { 0x13,  2, 1};

  const CmType          Cm_ClrError    = { 0x14,  0, 0};
  const CmType          Cm_GetError    = { 0x15,  0, 1};

  const CmType          Cm_GetIOBuf    = { 0x16,  0, 0xFF};
  const CmType          Cm_GetPIOName  = { 0x17,  1, 6};
  const CmType          Cm_GetMemCfg   = { 0x18,  0, 1};


// INDIRECT COMMANDS
//-------------------
  const CmType          Cm_IO_WriteEEByte  = { 0x80,  3, 0};
  const CmType          Cm_IO_ReadEEByte   = { 0x81,  2, 0};

  const CmType          Cm_IO_StupidReset  = { 0x82,  0, 0};
  const CmType          Cm_IO_StupidStep   = { 0x83,  0, 0};
  const CmType          Cm_IO_StupidStop   = { 0x84,  0, 0};
  const CmType          Cm_IO_StupidRun    = { 0x85,  0, 0};
  const CmType          Cm_IO_Reboot       = { 0x86,  0, 0};
  const CmType          Cm_IO_SetTime      = { 0x87,  3, 0};
  const CmType          Cm_IO_SetDate      = { 0x88,  4, 0};




void SetRTS()
//--------------------------------------------------------------------------}
{
  if (COM_Lock) return;
  ModemCtrl |= RTS;
  outp(UART_MCTRL, ModemCtrl);
}




void ClrRTS()
//--------------------------------------------------------------------------}
{
  ModemCtrl &=  (RTS ^ 0xFFFF);
  outp(UART_MCTRL, ModemCtrl);
}



void COM_Close()
//--------------------------------------------------------------------------}
{
  if (COM_Disable) return;
  outp(UART_IE, 0);
  outp(UART_MCTRL, 0);
  outp(UART_LCTRL, LINE_RXTX);
  inp(UART_DATA);
  inp(UART_LSTAT);
  inp(UART_MSTAT);
}




unsigned char HafHaf()
//--------------------------------------------------------------------------}
{
  // funkce vraci hodnotu ANO/NE  (1/0)

  #ifdef WDOGOFF
    return(0);
  #endif

  if (WDog < *p_BiosTime) return(1); else return(0);
}




void SetWD(unsigned long TimeDiff)
//--------------------------------------------------------------------------}
{
  WDog = *p_BiosTime + TimeDiff;
}





void TxComplete()
//--------------------------------------------------------------------------}
{
  unsigned char Tmp;


  if (COM_Lock) return;
  SetWD(STD_TIMEOUT);

  do {
    Tmp = (unsigned char)inp(UART_LSTAT);
    if ((Tmp & STAT_TXREADY) && (Tmp & STAT_TXEMPTY)) return;
  } while (!HafHaf());

  COM_Error = ERR_TIMEOUTTX;
  COM_Lock = 1;
  ClrRTS();
}




void SetBaudRate(unsigned long Baud)
//--------------------------------------------------------------------------}
{
  unsigned long Divisor;


  if (COM_Lock) return;
  COM_Error = ERR_NONE;
  TxComplete();
  ClrRTS();

  Divisor = 115200 / Baud;
  if (!Divisor)  Divisor = 115200;
  if (Divisor > 2047)  Divisor = 2047;

  outp(UART_LCTRL, LINE_DLAB);
  outp(UART_LDIV, (unsigned char)(Divisor % 256));

  outp(UART_HDIV, (unsigned char)(Divisor / 256));
  outp(UART_LCTRL, LINE_RXTX);
}




void COM_Init()
//-----------------------------------------------------------------}
{
  unsigned int   *p_BiosCOM = (unsigned int *)(0, 0x400);
  unsigned int   COM_Port;
  unsigned char  TempCom;
  unsigned char  Dummy;
  unsigned int   i;



  for (i=0; i<32; i++) StationInfo[i].OK = 0;
  TempCom = ComPort;
  COM_Error = ERR_NONE;
  COM_Lock = 0;
  if ((TempCom < 1) || (TempCom > 4)) {
    COM_Error = ERR_INITCOM;
    Success = 0;
    COM_Lock = 1;
    COM_Disable = 1;
    COM_Port  = 0x3FF;
    UART_DATA  = COM_Port;
    UART_LDIV  = COM_Port;
    UART_IE    = COM_Port;
    UART_HDIV  = COM_Port;
    UART_ID    = COM_Port;
    UART_LCTRL = COM_Port;
    UART_MCTRL = COM_Port;
    UART_LSTAT = COM_Port;
    UART_MSTAT = COM_Port;
    return;
  }

  TempCom--;
  COM_Port = p_BiosCOM[TempCom];

  switch (TempCom) {
    case 0: if (!COM_Port) COM_Port = 0x3F8;
    case 1: if (!COM_Port) COM_Port = 0x2F8;
    case 2: if (!COM_Port) COM_Port = 0x3E8;
    case 3: if (!COM_Port) COM_Port = 0x2E8;
  }

  UART_DATA  = COM_Port;
  UART_LDIV  = COM_Port;
  UART_IE    = COM_Port + 1;
  UART_HDIV  = COM_Port + 1;
  UART_ID    = COM_Port + 2;
  UART_LCTRL = COM_Port + 3;
  UART_MCTRL = COM_Port + 4;
  UART_LSTAT = COM_Port + 5;
  UART_MSTAT = COM_Port + 6;

  for (i=1; i<7; i++) outp((COM_Port + i), 0);

  COM_Disable = 0;
  SetBaudRate(BaudRate);
  ModemCtrl = DEFAULT_MCTRL;
  outp(UART_MCTRL, ModemCtrl);
  outp(UART_IE, 0);
  inp(UART_DATA);
  inp(UART_LSTAT);
  LineStatus = 0;
  LastLineStatus = 0;
  Success = 1;

}




void TxData(unsigned char Dato)
//-----------------------------------------------------------------}
{
  if (COM_Lock) return;
  SetWD(STD_TIMEOUT);

  do {
    if (inp(UART_LSTAT) & STAT_TXREADY) {
      outp(UART_DATA, Dato);
      if (BaudRate < 9600) delay(1);
      return;
    }
  } while (!HafHaf());

  COM_Error = ERR_TIMEOUTTX;
  COM_Lock = 1;
  ClrRTS();
}




void TxHead(unsigned char Dato)
//-----------------------------------------------------------------}
{
  if (COM_Lock) return;
  TxComplete();
  if (COM_Lock) return;
  outp(UART_LCTRL, LINE_TXHEAD);
  outp(UART_DATA, Dato);
  if (BaudRate < 9600) delay(1);
  TxComplete();
  outp(UART_LCTRL, LINE_RXTX);
}





unsigned char RxReady()
//-----------------------------------------------------------------}
{
  // funkce vraci hodnotu ANO/NE  (1/0)

  LineStatus |= inp(UART_LSTAT);
  if (LineStatus & STAT_RXREADY) return(1); else return(0);
}




unsigned char RxBit9()
//-----------------------------------------------------------------}
{
  if (LastLineStatus & STAT_ERRPARITY) return(1); else return(0);
}




unsigned char RxData()
//-----------------------------------------------------------------}
{
  unsigned char TmpRx;

  if (COM_Lock) return(0);

  SetWD(STD_TIMEOUT);
  do {
    if (RxReady()) {
      TmpRx = inp(UART_DATA);
      LastLineStatus = LineStatus | inp(UART_LSTAT);
      LineStatus = 0;
      return(TmpRx);
    }
  } while (!HafHaf());

  COM_Error = ERR_TIMEOUTRX;
  COM_Lock = 1;
  ClrRTS();
  return(0);
}



void RxClear()
//-----------------------------------------------------------------}
{
  if (COM_Disable) return;

  ClrRTS();
  inp(UART_DATA);
  LastLineStatus = 0;
  LineStatus = 0;
  COM_Error = ERR_NONE;
}





void ReceiveBlock(unsigned char RxNum)
//-----------------------------------------------------------------}
{
  unsigned char   i;
  unsigned char   FrameSum;
  unsigned char   Check;


  if (COM_Lock) return;
  FrameSum = 0;

  if (RxNum == 0xFF) {
    if (!IndirectLen) {
      COM_Error = ERR_DATASIZE;
      COM_Lock = 1;
      return;
    }
    RxNum = IndirectLen;
  }

  for (i=0; i<RxNum; i++) {
    IOData[i] = RxData();
    FrameSum = FrameSum + IOData[i];
    SetRTS();
    TxHead(HD_REQUEST | AddrTarget);
    ClrRTS();
  }

  Check = RxData();
  if ((COM_Error == ERR_NONE) && (Check != FrameSum)) {
    COM_Error = ERR_RXSUM;
    COM_Lock = 1;
  }
}





void SendCmd(CmType Cm)
//-----------------------------------------------------------------}
{
  unsigned char  FrameSum;
  unsigned char  Dato;
  unsigned char  Dummy;
  unsigned char  i;


  if (COM_Disable) {
    COM_Error = ERR_INITCOM;
    return;
  }

  RxClear();

  COM_Error = ERR_NONE;
  COM_Lock = 0;

  Dato = HD_CMDRQ | AddrTarget;         // ramec bude RQCMD + adresa
  FrameSum = Dato;

  SetRTS();                             // otevrit vysilac
  TxHead(Dato);                         // hlavicka ramce

  TxData(Cm.Cm);                        // COMMAND
  FrameSum = FrameSum + Cm.Cm;

                                        // tady se vysilaji prip. parametry
  if (Cm.Tx > 0) {
    for (i=0; i<Cm.Tx; i++) {
      TxData(IOData[i]);
      FrameSum += IOData[i];
    }
  }

  TxData(FrameSum);                     // kontrolni soucet
  TxComplete();                         // pockat na konec
  ClrRTS();                             // a zavrit hubu

  if (AddrTarget == 31) return;         // globalni adresa je bez odezvy !
  if (Cm.Cm >= 0x80) return;            // indirect CMD bez odezvy - zatim !

                                        // ramec s ERROR-RESPONSE }
  if (!Cm.Rx) {
    ReceiveBlock(1);
    if ((IOData[0]) && (COM_Error == ERR_NONE)) {
      COM_Error = 0x80 | IOData[0];
      COM_Lock = 1;
    }
  }
  else ReceiveBlock(Cm.Rx);             // nebo ramec s DATA-RESPONSE
}








void SelectStation(unsigned char StAddr)
//-----------------------------------------------------------------}
{
  Success = 1;
  if (StAddr > 31) {
    COM_Error = ERR_ADDRESS;
    AddrTarget = StAddr & 0x1F;
    Success = 0;
    return;
  }

  AddrTarget = StAddr;
}






unsigned char NetActive()
//-----------------------------------------------------------------}
{
  // funkce vraci hodnotu ANO/NE  (1/0)

  unsigned long  SaveBaud;
  unsigned char  Active;


  if (COM_Disable) {
    return(0);
  }

  SaveBaud = BaudRate;
  BaudRate = 57600;
  COM_Init();
  RxClear();

  SetWD(2);
  Active = 0;

  do {
    if (RxReady()) Active = 1;
  } while (!(HafHaf() || Active));

  RxClear();
  BaudRate = SaveBaud;
  COM_Init();
  return(Active);
}





void SendToken()
//-----------------------------------------------------------------}
{
  unsigned char  FrameSum;
  unsigned char  Dato;
  unsigned char  Rep;


  Success = 0;

  if (COM_Disable) {
    COM_Error = ERR_INITCOM;
    return;
  }
  Rep = 0;

  do {
    COM_Error = ERR_NONE;
    COM_Lock = 0;

    Dato = HD_TOKEN | AddrTarget;         // ramec bude TOKEN + adresa
    FrameSum = Dato;

    SetRTS();                             // otevrit vysilac
    TxHead(Dato);                         // hlavicka ramce

    TxData(31);                           // AddrTS = 31, je to blbost ale nevadi
    FrameSum += 31;

    TxData(FrameSum);                     // kontrolni soucet
    TxComplete();                         // pockat na konec
    ClrRTS();                             // a zavrit hubu
    Rep++;
  } while (!(NetActive() || (Rep > 4)));

  if (Rep <= 4) Success = 1;
}




void NetBreak()
//-----------------------------------------------------------------}
{
  unsigned char  Rep;


  Success = 0;
  if (COM_Disable) return;

  Rep = 3;

  do {
    Rep--;
    SetRTS();
    outp(UART_LCTRL, LINE_BREAK);
    delay(50);
    outp(UART_LCTRL, LINE_RXTX);
    ClrRTS();
    delay(5);
    if (! NetActive()) Rep=0;
  } while (Rep);

  if (Rep>0) Success = 1;
}




unsigned long BaudScan()
//--------------------------------------------------------------}
{
  unsigned char  i;
  unsigned char  Rep;
  unsigned char  BdFound;
  unsigned char  FirstHit;
  unsigned long  BdValue;



  BdFound = 0;
  BdValue = 0;
  COM_Error = ERR_NONE;

  NetBreak();

  for (i=0; i<BdNum; i++) {
    Rep = 2;
    FirstHit = 0;
    BaudRate = BdList[i];
    COM_Init();
    SelectStation(31);

    do {
      SendCmd(Cm_GetACK);
      if (COM_Error) {
        Success = 0;
        COM_Error = ERR_TIMEOUTTX;
        return(0);
      }
      RxData();

      if (COM_Error == ERR_TIMEOUTRX) Rep = 0;
      else {
        Rep--;
        if (! FirstHit) FirstHit = 1;
        else {
          BdValue = BdList[i];
          if (BdFound) {
            Success = 0;
            COM_Error = ERR_MOREBAUD;
            return(0);
          }
          BdFound = 1;
          Rep = 0;
        }
      }

    } while (Rep);
  }

  if (BdFound) {
    BaudRate = BdValue;
    Success = 1;
  }
  else {
    Success = 0;
    COM_Error = ERR_NETEMPTY;
    return(0);
  }
  return(BdValue);
}










void NetScan()
//--------------------------------------------------------------}
/*
  procedura prohleda vsechny adresy od 0 do 30, na polozku 0 do
  StationList zapise pocet aktivnich stanic a na dalsi polozky postupne
  zapise jejich adresy (ve vzestupnem poradi)
*/

{
  unsigned char   i;
  unsigned char   BdFound;
  unsigned char   FirstHit;
  unsigned int    Rep;
  unsigned char   Dummy;



  BaudScan();

  if (!Success) return;

  StationNum = 0;

  NetBreak();

  COM_Init();

  if (COM_Error) return;

  for (i=0; i<31; i++) {
    SelectStation(i);
    IOData[0] = 0;
    SendCmd(Cm_GetACK);

    if (COM_Error && (COM_Error != ERR_TIMEOUTRX)) {
      SendCmd(Cm_GetACK);
    }

    if (IOData[0] == 0x5A) {
      StationList[StationNum] = i;
      StationNum++;
    }
  }
  COM_Error = ERR_NONE;
  Success = 1;
}







unsigned char ReliableSend(CmType Cmd)
//------------------------------------------------------------}
{
  // funkce vraci hodnotu ANO/NE  (1/0)

  unsigned char    Rep;
  unsigned char    SaveIO[10];
  unsigned char    i;


  for (i=0; i<10; i++) SaveIO[i] = IOData[i];

  Rep = NUM_REPEATS;

  do {
    for (i=0; i<10; i++) IOData[i] = SaveIO[i];
    SendCmd(Cmd);
    if (!COM_Error) return(1);
    SendCmd(Cm_ClrError);
    Rep--;
  } while (Rep);
  return(0);
}



void UpdateInfo()
//------------------------------------------------------------}
{
  unsigned int  TmpWord;
  unsigned char TmpStr[10];
  char          *endptr;


  if (!ReliableSend(Cm_GetNETVer)) {
    Success = 0;
    return;
  }
  TmpStr[0] = IOData[0];
  TmpStr[1] = IOData[2];
  TmpStr[2] = IOData[3];
  TmpStr[3] = 0;
  TmpWord = strtoul(TmpStr, &endptr, 10);
  if (!TmpWord) {
    Success = 0;
    return;
  }
  #define STINFO StationInfo[AddrTarget]

  STINFO.StackAddr = 0x1800;
  STINFO.MemTop    = 0x1FFF;
  STINFO.FWVersion = TmpWord;

  if (TmpWord<338) {
    STINFO.OK = 1;
    Success = 1;
    return;
  }
  if (!ReliableSend(Cm_GetMemCfg)) {
    Success = 0;
    return;
  }
  if (IOData[0] & 0x80) {
    STINFO.MemTop = 0x73FF;
    STINFO.StackAddr = (IOData[0] & 0x7F) * 256;
  }
  STINFO.OK = 1;
  Success = 1;

  #undef STINFO
}






unsigned int ReadSTPWord(unsigned int Addr)
//--------------------------------------------------------------------------}
{
  unsigned int   Dato;


  IOData[0] = Addr % 256;
  if (!ReliableSend(Cm_ReadSWord)) {
    Dato = 0;
    Success = 0;
    return(Dato);
  }
  Dato = (256 * IOData[0]) + IOData[1];
  Success = 1;
  return(Dato);
}





unsigned char ReadSTPBit(unsigned int Addr)
//-------------------------------------------------------------------------}
{
  IOData[0] = Addr / 256;
  IOData[1] = Addr % 256;
  if (!ReliableSend(Cm_ReadSBit)) {
    Success = 0;
    return(0);
  }
  Success = 1;
  if (IOData[0]==0) return(0); else return(1);
}





void WriteSTPWord(unsigned int Addr, unsigned int Dato)
//--------------------------------------------------------------------------}
{
  IOData[0] = Addr % 256;
  IOData[1] = Dato / 256;
  IOData[2] = Dato % 256;

  if (!ReliableSend(Cm_WriteSWord)) {
    Success = 0;
    return;
  }

  Success = 1;
}




void WriteSTPBit(unsigned int Addr, unsigned char Bit)
//--------------------------------------------------------------------------}
{
  unsigned char  OK;


  IOData[0] = Addr / 256;
  IOData[1] = Addr % 256;
  if (Bit)  Success = ReliableSend(Cm_SetSBit); else Success = ReliableSend(Cm_ClrSBit);
}






unsigned char ReadXBlock(void *XBuffer, unsigned int Offset, unsigned int Size)
//------------------------------------------------------------------
// funkce vraci hodnotu ANO/NE  (1/0)
{
  unsigned int     i;
  unsigned char    ErrCnt;


  IOData[0] = Offset / 256;
  IOData[1] = Offset % 256;
  if (!ReliableSend(Cm_SetXPtr)) return(0);
  i = 0;

  do {
    SendCmd(Cm_ReadXPB);
    ((unsigned char *)XBuffer)[i] = IOData[0];

    if (COM_Error) {
      if (ErrCnt > NUM_REPEATS) return(0);
      SendCmd(Cm_ClrError);
      ErrCnt++;
      IOData[0] = (Offset + i) / 256;
      IOData[1] = (Offset + i) % 256;
      if (! ReliableSend(Cm_SetXPtr)) return(0);
    }
    else {
      i++;
      ErrCnt = 0;
    }
  } while (i < Size);
  return(1);
}




unsigned char WriteXBlock(void *XBuffer, unsigned int Offset, unsigned int Size)
//------------------------------------------------------------}
// funkce vraci hodnotu ANO/NE  (1/0)
{
  unsigned int     i;
  unsigned char    ErrCnt;


  IOData[0] = Offset / 256;
  IOData[1] = Offset % 256;
  if (! ReliableSend(Cm_SetXPtr)) return(0);
  i = 0;

  do {
    IOData[0] = ((unsigned char *)XBuffer)[i];
    SendCmd(Cm_WriteXPB);

    if (COM_Error) {
      if (ErrCnt > NUM_REPEATS) return(0);
      SendCmd(Cm_ClrError);
      ErrCnt++;
      IOData[0] = (Offset + i) / 256;
      IOData[1] = (Offset + i) % 256;
      if (! ReliableSend(Cm_SetXPtr)) return(0);
    }
    else {
      i++;
      ErrCnt = 0;
    }

  } while (i < Size);
  return(1);
}



unsigned int ReadStack(void *Buffer, unsigned int Position, unsigned int Count)
//------------------------------------------------------------}
{
  unsigned int     i;
  unsigned int     TmpWord;
  unsigned int     NumBytes;
  unsigned int     StackPtr;


  COM_Error = ERR_NONE;
  Success = 1;
  if (!Count) return(0);
  NumBytes = Count * 2;

  #define STINFO StationInfo[AddrTarget]

  if (!STINFO.OK) UpdateInfo();
  if (!STINFO.OK) {
    Success = 0;
    return(0);
  }
  StackPtr = STINFO.StackAddr + (Position * 2);
  if (StackPtr > STINFO.MemTop) {
    Success = 0;
    return(0);
  }
  if ((StackPtr + NumBytes) > STINFO.MemTop) {
    NumBytes = STINFO.MemTop - StackPtr + 1;
  }
  Count = NumBytes / 2;

  if (! ReadXBlock(Buffer, StackPtr, NumBytes)) Success = 0;

  for (i=0; i<Count; i++) {
    TmpWord = ((unsigned int *)Buffer)[i];
    ((unsigned int *)Buffer)[i] = (TmpWord / 256) + (256 * (TmpWord % 256));
  }
  return(Count);
  #undef STINFO
}





unsigned int WriteStack(void *Buffer, unsigned int Position, unsigned int Count)
//------------------------------------------------------------}
{
  unsigned int     i;
  unsigned int     TmpWord;
  unsigned int     NumBytes;
  unsigned int     StackPtr;


  COM_Error = ERR_NONE;
  Success = 1;
  if (!Count) return(0);
  NumBytes = Count * 2;

  #define STINFO StationInfo[AddrTarget]

  if (!STINFO.OK) UpdateInfo();
  if (!STINFO.OK) {
    Success = 0;
    return(0);
  }
  StackPtr = STINFO.StackAddr + (Position * 2);
  if (StackPtr > STINFO.MemTop) {
    Success = 0;
    return(0);
  }
  if ((StackPtr + NumBytes) > STINFO.MemTop) {
    NumBytes = STINFO.MemTop - StackPtr + 1;
  }
  Count = NumBytes / 2;

  for (i=0; i<Count; i++) {
    TmpWord = ((unsigned int *)Buffer)[i];
    ((unsigned int *)Buffer)[i] = (TmpWord / 256) + (256 * (TmpWord % 256));
  }

  if (! WriteXBlock(Buffer, StackPtr, NumBytes)) Success = 0;
  return(Count);

  #undef STINFO
}




char *GetErrorString(char *ErrString)
//-----------------------------------------------------------------}
{
  if (COM_Error >= 0x80) {
    COM_Error -= 0x80;

    if (COM_Error > MAX_TARGETERR) COM_Error = 0;
    strcpy(ErrString, ErrText_TPrefix);
    strcat(ErrString, ErrText_Target[COM_Error]);
  }
  else {
    if (COM_Error > MAX_INTERNERR) COM_Error = 0;
    strcpy(ErrString, ErrText_IPrefix);
    strcat(ErrString, ErrText_Internal[COM_Error]);
  }
  return(ErrString);
}






