unit PESNET3;


interface
{--------------------------------------------------------------------------}



var
  BaudRate     : longint;
  ComPort      : byte;
  COM_Error    : byte;
  StationNum   : byte;
  StationList  : array[0..31] of byte;
  Success      : boolean;



procedure COM_Init;
procedure COM_Close;
procedure SendToken;
procedure SelectStation(StAddr : byte);
procedure NetBreak;
function  BaudScan:longint;
procedure NetScan;
function  ReadSTPWord(Addr:word):word;
function  ReadSTPBit(Addr:word):boolean;
procedure WriteSTPWord(Addr:word; Dato:word);
procedure WriteSTPBit(Addr:word; Bit:boolean);

function  ReadStack(var Buffer; Position : word; Count : word):word;
function  WriteStack(var Buffer; Position : word; Count : word):word;

function  GetErrorString : string;
procedure UpdateInfo;


const
  ERR_NONE          : byte = $00;
  ERR_INITCOM       : byte = $01;
  ERR_TIMEOUT       : byte = $02;
  ERR_TIMEOUTTX     : byte = $03;
  ERR_TIMEOUTRX     : byte = $04;
  ERR_RXSUM         : byte = $05;
  ERR_DATASIZE      : byte = $06;
  ERR_ADDRESS       : byte = $07;
  ERR_DEBUGFLAG     : byte = $08;
  ERR_MOREBAUD      : byte = $09;
  ERR_NETEMPTY      : byte = $0A;

  ERR_BADERROR      : byte = $80;
  ERR_BADCMD        : byte = $81;
  ERR_TARGETSUM     : byte = $82;
  ERR_BADFRAME      : byte = $83;
  ERR_SELFRX        : byte = $84;
  ERR_IOFULL        : byte = $85;
  ERR_NODATA        : byte = $86;





implementation
{--------------------------------------------------------------------------}
uses crt;


type
  CmType = record
    Cm : byte;
    Tx : byte;
    Rx : byte;
  end;

  InfoType = record
    OK         : boolean;
    FWVersion  : word;
    StackAddr  : word;
    MemTop     : word;
  end;


var
  BiosTime     : longint absolute $0:$046C;
  WDog         : longint;


  ModemCtrl    : byte;

  COM_Lock     : boolean;
  COM_Disable  : boolean;

  LineStatus   : byte;
  LastLineStatus : byte;

  UART_DATA    : word;
  UART_LDIV    : word;
  UART_IE      : word;
  UART_HDIV    : word;
  UART_ID      : word;
  UART_LCTRL   : word;
  UART_MCTRL   : word;
  UART_LSTAT   : word;
  UART_MSTAT   : word;

  AddrTarget   : byte;

  IOData       : array[0..20] of byte;
  IndirectLen  : byte;

  i            : word;

  StationInfo  : array[0..31] of InfoType;






const
  MAX_INTERNERR     : byte = 10;
  MAX_TARGETERR     : byte = 6;

  ErrText_IPrefix   : string[20] = 'INTERNAL ERROR: ';
  ErrText_TPrefix   : string[20] = '  TARGET ERROR: ';

  ErrText_Internal  : array[0..10] of string[30] = (
        'UNSUPPORTED ERROR CODE',
        'COM-PORT INIT ERROR',
        'GENERAL TIME-OUT ON COM',
        'TX TIME-OUT ON COM OVERFLOW',
        'RX TIME-OUT ON COM OVERFLOW',
        'CHECKSUM ERROR ON RX DATA',
        'RX BLOCK LENGTH NOT SPECIFIED',
        'STATION ADDRESS OUT OF RANGE',
        'DEBUG - MARKING FLAG',
        'MULTIPLE BAUD RATES ON NETWORK',
        'NO STATIONS ON NETWORK'
        );

  ErrText_Target    : array[0..6] of string[30] = (
        'UNSUPPORTED ERROR CODE',
        'UNKNOWN COMMAND IN CMD-FRAME',
        'CHECKSUM ERROR IN LAST FRAME',
        'UNKNOWN TYPE OF FRAME',
        'TRANSMITT SELF-CHECK ERROR',
        'INPUT BUFFER FULL - FRAME LOST',
        'IO BUFFER EMPTY !'
        );


  STD_TIMEOUT          : byte = 2;      {standard-timeout asi 55-160 ms}

  LINE_RXTX            : byte = $3F;
  LINE_TXHEAD          : byte = $37;
  LINE_DLAB            : byte = $DD;
  LINE_BREAK           : byte = $7D;

  DOUBLESTOP           : byte = $04;
  PAR_ENABLE           : byte = $08;
  PAR_EVEN             : byte = $10;
  PAR_NULL             : byte = $20;
  BREAK                : byte = $40;
  DLAB                 : byte = $80;


  DEFAULT_MCTRL        : byte = $01;

  DTR                  : byte = $01;
  RTS                  : byte = $02;
  IRQ_ENABLE           : byte = $08;


  STAT_RXREADY         : byte = $01;
  STAT_OVERRUN         : byte = $02;
  STAT_ERRPARITY       : byte = $04;
  STAT_ERRFRAMING      : byte = $08;
  STAT_TXREADY         : byte = $20;
  STAT_TXEMPTY         : byte = $40;

  HD_CMDRQ             : byte = $A0;
  HD_CMDF              : byte = $80;
  HD_TOKEN             : byte = $C0;
  HD_REQUEST           : byte = $E0;




const
  BdNum  : byte = 4;
  BdList : array[0..3] of longint = (2400, 9600, 19200, 57600);


  NUM_REPEATS      : byte = 3;
  STPBUFFER_SIZE   : word = 4096;
  STUPID_BEGIN     : word = $0400;
  ACK_KEYWORD      : byte = $5A;



{ DIRECT COMMANDS }
{-----------------}
  Cm_GetACK:        CmType = (Cm: $01; Tx: 0; Rx:1);
  Cm_GetStatus:     CmType = (Cm: $02; Tx: 0; Rx:1);
  Cm_GetNETVer:     CmType = (Cm: $03; Tx: 0; Rx:6);
  Cm_GetSTPVer:     CmType = (Cm: $04; Tx: 0; Rx:6);

  Cm_WriteXByte:    CmType = (Cm: $05; Tx: 3; Rx:0);
  Cm_ReadXByte:     CmType = (Cm: $06; Tx: 2; Rx:1);

  Cm_Abort:         CmType = (Cm: $07; Tx: 0; Rx:0);

  Cm_SetXPtr:       CmType = (Cm: $08; Tx: 2; Rx:0);

  Cm_WriteXPB:      CmType = (Cm: $09; Tx: 1; Rx:0);
  Cm_WriteXPW:      CmType = (Cm: $0A; Tx: 2; Rx:0);
  Cm_WriteXPDW:     CmType = (Cm: $0B; Tx: 4; Rx:0);

  Cm_ReadXPB:       CmType = (Cm: $0C; Tx: 0; Rx:1);
  Cm_ReadXPW:       CmType = (Cm: $0D; Tx: 0; Rx:2);
  Cm_ReadXPDW:      CmType = (Cm: $0E; Tx: 0; Rx:4);

  Cm_WriteSWord:    CmType = (Cm: $0F; Tx: 3; Rx:0);
  Cm_ReadSWord:     CmType = (Cm: $10; Tx: 1; Rx:2);

  Cm_SetSBit:       CmType = (Cm: $11; Tx: 2; Rx:0);
  Cm_ClrSBit:       CmType = (Cm: $12; Tx: 2; Rx:0);
  Cm_ReadSBit:      CmType = (Cm: $13; Tx: 2; Rx:1);

  Cm_ClrError:      CmType = (Cm: $14; Tx: 0; Rx:0);
  Cm_GetError:      CmType = (Cm: $15; Tx: 0; Rx:1);

  Cm_GetIOBuf:      CmType = (Cm: $16; Tx: 0; Rx:$FF);

  Cm_GetPIOName:    CmType = (Cm: $17; Tx: 1; Rx:6);
  Cm_GetMemCfg:     CmType = (Cm: $18; Tx: 0; Rx:1);



{ INDIRECT COMMANDS }
{-------------------}
  CmIO_WriteEEByte: CmType = (Cm: $80; Tx: 3; Rx:0);
  CmIO_ReadEEByte:  CmType = (Cm: $81; Tx: 2; Rx:0);

  CmIO_StupidReset: CmType = (Cm: $82; Tx: 0; Rx:0);
  CmIO_StupidStep:  CmType = (Cm: $83; Tx: 0; Rx:0);
  CmIO_StupidStop:  CmType = (Cm: $84; Tx: 0; Rx:0);
  CmIO_StupidRun:   CmType = (Cm: $85; Tx: 0; Rx:0);
  CmIO_Reboot:      CmType = (Cm: $86; Tx: 0; Rx:0);
  CmIO_SetTime:     CmType = (Cm: $87; Tx: 3; Rx:0);
  CmIO_SetDate:     CmType = (Cm: $88; Tx: 4; Rx:0);





procedure SetRTS;
{--------------------------------------------------------------------------}
begin
  if COM_Lock then exit;
  ModemCtrl := ModemCtrl or RTS;
  port[UART_MCTRL] := ModemCtrl;
end;



procedure ClrRTS;
{--------------------------------------------------------------------------}
begin
  ModemCtrl := ModemCtrl and (not RTS);
  port[UART_MCTRL] := ModemCtrl;
end;



procedure COM_Close;
{--------------------------------------------------------------------------}
var
  Dummy : byte;
begin
  if COM_Disable then exit;
  port[UART_IE] := 0;
  port[UART_MCTRL] := 0;
  port[UART_LCTRL] := LINE_RXTX;
  Dummy := port[UART_DATA];
  Dummy := port[UART_LSTAT];
  Dummy := port[UART_MSTAT];
end;



function HafHaf:boolean;
{--------------------------------------------------------------------------}
begin

{$IFDEF WDOGOFF}
  HafHaf:=false;
  exit;
{$ENDIF}

  if WDog < BiosTime
    then HafHaf:=true
    else HafHaf:=false;
end;



procedure SetWD(TimeDiff: longint);
{--------------------------------------------------------------------------}
begin
  WDog:=BiosTime + TimeDiff;
end;


(*
procedure UserWD(var WDTimer: longint; TimeDiff: longint);
{--------------------------------------------------------------------------}
begin
  WDTimer:=BiosTime + TimeDiff;
end;



function UserHaf(WDTimer: longint):boolean;
{--------------------------------------------------------------------------}
begin

{$IFDEF WDOGOFF}
  UserHaf:=false;
  exit;
{$ENDIF}

  if WDTimer < BiosTime
    then UserHaf:=true
    else UserHaf:=false;
end;
*)




procedure TxComplete;
{--------------------------------------------------------------------------}
var
  Tmp : byte;
begin
  if COM_Lock then exit;
  SetWD(STD_TIMEOUT);

  repeat
    Tmp := port[UART_LSTAT];
    if ((Tmp and STAT_TXREADY)<>0) and ((Tmp and STAT_TXEMPTY)<>0) then exit
  until HafHaf;

  COM_Error := ERR_TIMEOUTTX;
  COM_Lock := true;
  ClrRTS;
end;





procedure SetBaudRate(Baud : longint);
{--------------------------------------------------------------------------}
var
  Divisor     : longint;

begin
  if COM_Lock then exit;
  COM_Error := ERR_NONE;
  TxComplete;
  ClrRTS;

  Divisor := 115200 div Baud;
  if (Divisor = 0) then  Divisor := 115200;
  if (Divisor > 2047) then Divisor := 2047;

  port[UART_LCTRL] := LINE_DLAB;
  port[UART_LDIV] := byte(Divisor mod 256);
  port[UART_HDIV] := byte(Divisor div 256);
  port[UART_LCTRL] := LINE_RXTX;
end;



procedure COM_Init;
{-----------------------------------------------------------------}
var
  BiosCOM      : array[0..3] of word absolute $0:$400;
  COM_Port     : word;
  TempCom      : byte;
  Dummy        : byte;
  i            : byte;

begin
  TempCom := ComPort;
  COM_Error := ERR_NONE;
  COM_Lock := false;
  if (TempCom < 1) or (TempCom > 4) then begin
    COM_Error := ERR_INITCOM;
    Success:=false;
    COM_Lock := true;
    COM_Disable := true;
    COM_Port  := $3FF;
    UART_DATA  := COM_Port;
    UART_LDIV  := COM_Port;
    UART_IE    := COM_Port;
    UART_HDIV  := COM_Port;
    UART_ID    := COM_Port;
    UART_LCTRL := COM_Port;
    UART_MCTRL := COM_Port;
    UART_LSTAT := COM_Port;
    UART_MSTAT := COM_Port;
    exit;
  end;

  dec(TempCom);
  COM_Port := BiosCOM[TempCom];

  case TempCom of
    0: if (COM_Port = 0) then COM_Port := $3F8;
    1: if (COM_Port = 0) then COM_Port := $2F8;
    2: if (COM_Port = 0) then COM_Port := $3E8;
    3: if (COM_Port = 0) then COM_Port := $2E8;
  end; {case}

  UART_DATA  := COM_Port;
  UART_LDIV  := COM_Port;
  UART_IE    := COM_Port + 1;
  UART_HDIV  := COM_Port + 1;
  UART_ID    := COM_Port + 2;
  UART_LCTRL := COM_Port + 3;
  UART_MCTRL := COM_Port + 4;
  UART_LSTAT := COM_Port + 5;
  UART_MSTAT := COM_Port + 6;

  for i:=1 to 6 do port[COM_Port + i] := 0;

  COM_Disable := false;
  SetBaudRate(BaudRate);
  ModemCtrl := DEFAULT_MCTRL;
  port[UART_MCTRL] := ModemCtrl;
  port[UART_IE] := $00;
  Dummy := port[UART_DATA];
  Dummy := port[UART_LSTAT];
  LineStatus := 0;
  LastLineStatus := 0;
  Success:=true;

end;



procedure TxData(Dato : byte);
{-----------------------------------------------------------------}
begin
  if COM_Lock then exit;
  SetWD(STD_TIMEOUT);

  repeat
    if ((port[UART_LSTAT] and STAT_TXREADY)<>0) then begin
      port[UART_DATA] := Dato;
      if (BaudRate < 9600) then delay(1);
      exit;
    end;
  until HafHaf;

  COM_Error := ERR_TIMEOUTTX;
  COM_Lock := true;
  ClrRTS;
end;



procedure TxHead(Dato : byte);
{-----------------------------------------------------------------}
begin
  if COM_Lock then exit;
  TxComplete;
  if COM_Lock then exit;
  port[UART_LCTRL] := LINE_TXHEAD;
  port[UART_DATA] := Dato;
  if (BaudRate < 9600) then delay(1);
  TxComplete;
  port[UART_LCTRL] := LINE_RXTX;
end;



function RxReady : boolean;
{-----------------------------------------------------------------}
begin
  LineStatus := LineStatus or port[UART_LSTAT];
  if (LineStatus and STAT_RXREADY)<>0 then RxReady := true else RxReady := false;

end;



function RxBit9 : boolean;
{-----------------------------------------------------------------}
begin
  if (LastLineStatus and STAT_ERRPARITY) <> 0 then RxBit9 := true
    else RxBit9 := false;
end;




function RxData : byte;
{-----------------------------------------------------------------}
begin
  if COM_Lock then exit;

  SetWD(STD_TIMEOUT);
  repeat
    if RxReady then begin
      RxData := port[UART_DATA];
      LastLineStatus := LineStatus or port[UART_LSTAT];
      LineStatus := 0;
      exit;
    end;
  until HafHaf;

  COM_Error := ERR_TIMEOUTRX;
  COM_Lock := true;
  ClrRTS;
  RxData := 0;
end;




procedure RxClear;
{-----------------------------------------------------------------}
var
  Data : byte;
begin
  if COM_Disable then exit;

  ClrRTS;
  Data := port[UART_DATA];
  LastLineStatus := 0;
  LineStatus := 0;
  COM_Error := ERR_NONE;
end;





procedure ReceiveBlock(RxNum : byte);
{-----------------------------------------------------------------}
var
  i         : byte;
  FrameSum  : byte;
  Check     : byte;

begin
  if COM_Lock then exit;
  FrameSum := 0;

  if (RxNum = $FF) then begin
    if (IndirectLen = 0) then begin
      COM_Error := ERR_DATASIZE;
      COM_Lock := true;
      exit;
    end;
    RxNum := IndirectLen;
  end;

  {$R-}

  for i:=0 to (RxNum-1) do begin
    IOData[i] := RxData;
    FrameSum := FrameSum + IOData[i];
    SetRTS;
    TxHead(HD_REQUEST or AddrTarget);
    ClrRTS;
  end;

 {$R+}

  Check := RxData;
  if (COM_Error = ERR_NONE) and (Check <> FrameSum) then begin
    COM_Error := ERR_RXSUM;
    COM_Lock := true;
  end;
end;







procedure SendCmd(Cm : CmType);
{-----------------------------------------------------------------}
var
  FrameSum      : byte;
  Dato          : byte;
  i             : byte;
  Dummy         : byte;

begin
  if COM_Disable then begin
    COM_Error := ERR_INITCOM;
    exit;
  end;

  RxClear;

  COM_Error := ERR_NONE;
  COM_Lock := false;

  {$R-}

  Dato := HD_CMDRQ or AddrTarget;       { ramec bude RQCMD + adresa }
  FrameSum := Dato;

  SetRTS;                               { otevrit vysilac }
  TxHead(Dato);                         { hlavicka ramce }

  TxData(Cm.Cm);                        { COMMAND }
  FrameSum := FrameSum + Cm.Cm;

  if (Cm.Tx > 0) then begin             { tady se vysilaji prip. parametry }
    for i:=0 to (Cm.Tx-1) do begin
      TxData(IOData[i]);
      FrameSum := FrameSum + IOData[i];
    end;
  end;

  TxData(FrameSum);                     { kontrolni soucet }
  TxComplete;                           { pockat na konec }
  ClrRTS;                               { a zavrit hubu }

  {$R+}

  if (AddrTarget = 31) then exit;       { globalni adresa je bez odezvy ! }
  if (Cm.Cm >= $80) then exit;          { indirect CMD bez odezvy - zatim ! }

  if (Cm.Rx = 0) then begin             { ramec s ERROR-RESPONSE }
    ReceiveBlock(1);
    if (IOData[0] <> 0) and (COM_Error = ERR_NONE) then begin
      COM_Error := $80 or IOData[0];
      COM_Lock := true;
    end
  end
  else ReceiveBlock(Cm.Rx);             { nebo ramec s DATA-RESPONSE }
end;











procedure SelectStation(StAddr : byte);
{-----------------------------------------------------------------}
begin
  Success:=true;
  if (StAddr > 31) then begin
    COM_Error := ERR_ADDRESS;
    AddrTarget := StAddr and $1F;
    Success:=false;
    exit;
  end;

  AddrTarget := StAddr;
end;




function NetActive : boolean;
{-----------------------------------------------------------------}
var
  SaveBaud  : longint;
  Active    : boolean;
begin
  if COM_Disable then begin
    NetActive := false;
    exit;
  end;

  SaveBaud := BaudRate;
  BaudRate := 57600;
  COM_Init;
  RxClear;

  SetWD(2);
  Active:=false;

  repeat
    if RxReady then Active:=true;
  until HafHaf or Active;

  RxClear;
  BaudRate := SaveBaud;
  COM_Init;
  NetActive := Active;
end;





procedure SendToken;
{-----------------------------------------------------------------}
var
  FrameSum      : byte;
  Dato          : byte;
  Rep           : byte;

begin
  Success:=false;
  if COM_Disable then begin
    COM_Error := ERR_INITCOM;
    exit;
  end;
  Rep:=0;

  repeat

    COM_Error := ERR_NONE;
    COM_Lock := false;

    Dato := HD_TOKEN or AddrTarget;       { ramec bude TOKEN + adresa }
    FrameSum := Dato;

    SetRTS;                               { otevrit vysilac }
    TxHead(Dato);                         { hlavicka ramce }

    TxData(31);                           {AddrTS = 31, je to blbost ale nevadi}
    FrameSum := FrameSum + 31;

    TxData(FrameSum);                     { kontrolni soucet }
    TxComplete;                           { pockat na konec }
    ClrRTS;                               { a zavrit hubu }
    inc(Rep);

  until (NetActive or (Rep > 4));

  if (Rep <= 4) then begin
    Success:=true;
  end;
end;





procedure NetBreak;
{-----------------------------------------------------------------}
var
  Rep   : byte;
begin
  Success:=false;
  if COM_Disable then exit;

  Rep:=3;
  repeat
    dec(Rep);
    SetRTS;
    port[UART_LCTRL] := LINE_BREAK;
    delay(50);
    port[UART_LCTRL] := LINE_RXTX;
    ClrRTS;
    delay(5);
    if not NetActive then Rep:=0;
  until Rep=0;

    if Rep>0 then Success:=true;
end;





function BaudScan : longint;
{--------------------------------------------------------------}
var
  i          : byte;
  Rep        : byte;
  BdFound    : boolean;
  FirstHit   : boolean;
  BdValue    : longint;
  Dummy      : byte;

begin
  BdFound:=false;
  BdValue:=0;
  COM_Error:=ERR_NONE;

  NetBreak;

  for i:=0 to (BdNum-1) do begin
    Rep:=2;
    FirstHit:=false;
    BaudRate:=BdList[i];
    COM_Init;
    SelectStation(31);

    repeat

      SendCmd(Cm_GetACK);
      if (COM_Error <> 0) then begin
        BaudScan:=0;
        Success:=false;
        COM_Error:=ERR_TIMEOUTTX;
        exit;
      end;
      Dummy := RxData;

      if (COM_Error=ERR_TIMEOUTRX) then Rep:=0 else begin
        dec(Rep);
        if (not FirstHit) then FirstHit:=true else begin
          BdValue:=BdList[i];
          if BdFound then begin
            BaudScan:=0;
            Success:=false;
            COM_Error:=ERR_MOREBAUD;
            exit;
          end;
          BdFound:=true;
          Rep:=0;
        end;
      end;

    until Rep=0;

  end;

  if BdFound then begin
    BaudRate:=BdValue;
    BaudScan:=BdValue;
    Success:=true;
  end
  else begin
    BaudScan:=0;
    Success:=false;
    COM_Error:=ERR_NETEMPTY;
  end;

end;










procedure NetScan;
{--------------------------------------------------------------}
(*
  procedura prohleda vsechny adresy od 0 do 30, na polozku 0 do
  StationList zapise pocet aktivnich stanic a na dalsi polozky postupne
  zapise jejich adresy (ve vzestupnem poradi)
*)

var
  i          : byte;
  BdFound    : boolean;
  FirstHit   : boolean;
  Rep        : word;
  Dummy      : byte;

begin
  BaudScan;

  if not Success then exit;

  StationNum := 0;

  NetBreak;

  COM_Init;

  if (COM_Error <> 0) then exit;

  for i:=0 to 30 do begin
    SelectStation(i);
    IOData[0] := 0;
    SendCmd(Cm_GetACK);

    if (COM_Error<>0) and (COM_Error<>ERR_TIMEOUTRX) then begin
      SendCmd(Cm_GetACK);
    end;

    if (IOData[0] = $5A) then begin
      StationList[StationNum] := i;
      inc(StationNum);
    end;
  end;
  COM_Error := ERR_NONE;
  Success:=true;
end;







function ReliableSend(Cmd : CmType):boolean;
{------------------------------------------------------------}
var
  Rep     : byte;
  SaveIO  : array[0..9] of byte;
  i       : byte;
begin
  for i:=0 to 9 do SaveIO[i]:=IOData[i];
  Rep:=NUM_REPEATS;
  repeat
    for i:=0 to 9 do IOData[i]:=SaveIO[i];
    SendCmd(Cmd);
    if (COM_Error=0) then begin
      ReliableSend:=true;
      exit;
    end;
    SendCmd(Cm_ClrError);
    dec(Rep);
  until Rep=0;
  ReliableSend:=false;
end;






procedure UpdateInfo;
{--------------------------------------------------------------------------}
var
  er      : integer;
  TmpStr  : string;
  TmpWord : word;

begin
  if not ReliableSend(Cm_GetNETVer) then begin
    Success:=false;
    exit;
  end;
  TmpStr:=chr(IOData[0]) + chr(IOData[2]) + chr(IOData[3]);
  Val(TmpStr, TmpWord, er);
  if er<>0 then begin
    Success:=false;
    exit;
  end;


  with StationInfo[AddrTarget] do begin
    StackAddr:=$1800;
    MemTop:=$1FFF;

    FWVersion:=TmpWord;
    if (TmpWord<338) then begin
      OK:=true;
      Success:=true;
      exit;
    end;
    if not ReliableSend(Cm_GetMemCfg) then begin
      Success:=false;
      exit;
    end;
    if (IOData[0] and $80)=$80 then begin
      MemTop:=$73FF;
      StackAddr:=(IOData[0] and $7F) * 256;
    end;
    OK:=true;
  end;
  Success:=true;

end;





function ReadSTPWord(Addr:word):word;
{--------------------------------------------------------------------------}
var
  Dato   : word;
begin
  IOData[0]:=Lo(Addr);
  if not ReliableSend(Cm_ReadSWord) then begin
    Dato:=0;
    Success:=false;
    ReadSTPWord:=Dato;
    exit;
  end;
  Dato:=(256 * IOData[0]) + IOData[1];
  Success:=true;
  ReadSTPWord:=Dato;
end;




function ReadSTPBit(Addr:word):boolean;
{--------------------------------------------------------------------------}
begin
  IOData[0]:=Hi(Addr);
  IOData[1]:=Lo(Addr);
  if not ReliableSend(Cm_ReadSBit) then begin
    Success:=false;
    ReadSTPBit:=false;
    exit;
  end;
  Success:=true;
  if IOData[0]=0 then ReadSTPBit:=false else ReadSTPBit:=true;
end;




procedure WriteSTPWord(Addr:word; Dato:word);
{--------------------------------------------------------------------------}
begin
  IOData[0]:=Lo(Addr);
  IOData[1]:=Hi(Dato);
  IOData[2]:=Lo(Dato);

  if not ReliableSend(Cm_WriteSWord) then begin
    Success:=false;
    exit;
  end;

  Success:=true;
end;




procedure WriteSTPBit(Addr:word; Bit:boolean);
{--------------------------------------------------------------------------}
var
  OK   : boolean;

begin
  IOData[0]:=Hi(Addr);
  IOData[1]:=Lo(Addr);
  if Bit then OK:=ReliableSend(Cm_SetSBit) else OK:=ReliableSend(Cm_ClrSBit);
  if not OK then Success:=false else Success:=true;
end;







function ReadXBlock(var XBuffer; Offset,Size : word):boolean;
{------------------------------------------------------------}
type
  Xarray = array[0..MaxInt] of byte;
var
  i     : word;
  ErrCnt: byte;

begin
  IOData[0]:=Hi(Offset);
  IOData[1]:=Lo(Offset);
  if not ReliableSend(Cm_SetXPtr) then begin
    ReadXBlock:=false;
    exit;
  end;
  i:=0;

  repeat

    SendCmd(Cm_ReadXPB);
    Xarray(XBuffer)[i]:=IOData[0];

    if (COM_Error <> 0) then begin
      if ErrCnt > NUM_REPEATS then begin
        ReadXBlock:=false;
        exit;
      end;
      SendCmd(Cm_ClrError);
      inc(ErrCnt);
      IOData[0]:=Hi(Offset + i);
      IOData[1]:=Lo(Offset + i);
      if not ReliableSend(Cm_SetXPtr) then begin
        ReadXBlock:=false;
        exit;
      end;
    end
    else begin
      inc(i);
      ErrCnt:=0;
    end;

  until not (i < Size);
  ReadXBlock:=true;
end;




function WriteXBlock(var XBuffer; Offset,Size : word):boolean;
{------------------------------------------------------------}
type
  Xarray = array[0..MaxInt] of byte;
var
  i     : word;
  ErrCnt: byte;
begin
  IOData[0]:=Hi(Offset);
  IOData[1]:=Lo(Offset);
  if not ReliableSend(Cm_SetXPtr) then begin
    WriteXBlock:=false;
    exit;
  end;
  i:=0;

  repeat

    IOData[0]:=Xarray(XBuffer)[i];
    SendCmd(Cm_WriteXPB);

    if (COM_Error <> 0) then begin
      if ErrCnt > NUM_REPEATS then begin
        WriteXBlock:=false;
        exit;
      end;
      SendCmd(Cm_ClrError);
      inc(ErrCnt);
      IOData[0]:=Hi(Offset + i);
      IOData[1]:=Lo(Offset + i);
      if not ReliableSend(Cm_SetXPtr) then begin
        WriteXBlock:=false;
        exit;
      end;
    end
    else begin
      inc(i);
      ErrCnt:=0;
    end;

  until not (i < Size);
  WriteXBlock:=true;
end;




function ReadStack(var Buffer; Position : word; Count : word):word;
{-------------------------------------------------------------------}
type
  Xarray = array[0..MaxInt] of byte;
  Warray = array[0..16383] of word;

var
  TmpWord    : word;
  i          : word;
  NumBytes   : word;
  StackPtr   : word;

begin
  COM_Error := ERR_NONE;
  Success := true;
  if (Count=0) then begin
    ReadStack:=0;
    exit;
  end;
  NumBytes:=Count*2;

  with StationInfo[AddrTarget] do begin
    if not OK then UpdateInfo;
    if not OK then begin
      Success:=false;
      ReadStack:=0;
      exit;
    end;

    StackPtr:=StackAddr + (Position * 2);
    if (StackPtr > MemTop) then begin
      Success:=false;
      ReadStack:=0;
      exit;
    end;

    if ((StackPtr + NumBytes) > MemTop) then NumBytes:=MemTop-StackPtr+1;
  end;
  Count:=NumBytes div 2;

  if not ReadXBlock(Buffer, StackPtr, NumBytes)
    then Success := false;

  for i:=0 to Count-1 do begin
    TmpWord := Warray(Buffer)[i];
    Warray(Buffer)[i] := Hi(TmpWord) + (256 * Lo(TmpWord));
  end;
  ReadStack:=Count;
end;




function WriteStack(var Buffer; Position : word; Count : word):word;
{------------------------------------------------------------}
type
  Xarray = array[0..MaxInt] of byte;
  Warray = array[0..1023] of word;

var
  TmpWord    : word;
  i          : word;
  NumBytes   : word;
  StackPtr   : word;

begin
  COM_Error := ERR_NONE;
  Success := true;

  if (Count=0) then begin
    WriteStack:=0;
    exit;
  end;
  NumBytes:=Count*2;

  with StationInfo[AddrTarget] do begin
    if not OK then UpdateInfo;
    if not OK then begin
      Success:=false;
      WriteStack:=0;
      exit;
    end;

    StackPtr:=StackAddr + (Position * 2);

    if (StackPtr + NumBytes) > MemTop then NumBytes:=MemTop-StackPtr+1;
  end;
  Count:=NumBytes div 2;

  for i:=0 to Count-1 do begin
    TmpWord := Warray(Buffer)[i];
    Warray(Buffer)[i] := Hi(TmpWord) + (256 * Lo(TmpWord));
  end;

  if not WriteXBlock(Buffer, StackPtr, NumBytes)
    then Success := false;
  WriteStack:=Count;
end;




function GetErrorString : string;
{-----------------------------------------------------------------}
begin
  if (COM_Error >= $80) then begin
    COM_Error := COM_Error - $80;
    if COM_Error > MAX_TARGETERR then COM_Error := 0;
    GetErrorString := ErrText_TPrefix + ErrText_Target[COM_Error];
  end else begin
    if COM_Error > (MAX_INTERNERR) then COM_Error := 0;
    GetErrorString := ErrText_IPrefix + ErrText_Internal[COM_Error];
  end;
end;






begin

  BaudRate:=57600;
  ComPort:=1;
  ModemCtrl := DEFAULT_MCTRL;
  COM_Lock := true;
  COM_Disable := true;
  AddrTarget := 0;
  IndirectLen := 0;
  Success := false;
  for i:=0 to 31 do StationList[i] := 0;
  for i:=0 to 31 do StationInfo[i].OK := false;
  StationNum := 0;

end.

